from ..twitter import twitter_session, load_tweets

_session = None
_config = None


def init(config, provided_session=None):
    global _config, _session
    _config = config
    if provided_session is not None:
        _session = provided_session


def get_session():
    global _session
    if _session is None:
        _session = twitter_session(_config)
    return _session


def get_tweets(query, result_type, since_id=None):
    if result_type not in ['recent', 'mixed', 'popular']:
        result_type = 'recent'
    return load_tweets(get_session(), _config, query, result_type, since_id=since_id)


import configparser
import queue
from . import PrintWorker, WallWorker
import os


__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))


def run_console(query, count, interval, result_type, **kwargs):
    if not os.path.exists(kwargs['auth_file']):
        raise FileNotFoundError('Authentication file doesn\'t exist. I don\'t have Twitter OAuth data.')

    config = _read_config(kwargs['auth_file'])

    print('Starting the Twitter Wall with query {}. Press Q<ENTER> to exit.'.format(query))

    q = queue.Queue()
    tweet_worker = WallWorker(config, q, query, interval, result_type, count)
    print_worker = PrintWorker(config, q)

    tweet_worker.start()
    print_worker.start()

    inp = None
    while inp != 'Q':
        inp = input()

    tweet_worker.stop()
    print_worker.stop()


def _read_config(config_file_path):
    config = configparser.ConfigParser()
    config.read([config_file_path, os.path.join(__location__, '../settings.cfg')])
    return config



import click
from .web import create_app
from .console import run_console


@click.group()
def cli():
    pass


@cli.command()
@click.option('--auth-file', default='auth.cfg', help='The authentication file location.')
@click.option('--debug/--no-debug', default=False, help='Run Flask in debug mode')
def webapp(**kwargs):
    app = create_app(kwargs['auth_file'], kwargs['debug'])
    app.run()


@cli.command()
@click.argument('query')
@click.option('--auth-file', default='auth.cfg', help='The authentication file location.')
@click.option(
    '--count', '-c',
    default=10,
    type=click.IntRange(0, 100),
    help='Number of tweets to load at startup.')
@click.option(
    '--interval', '-i',
    default=4,
    type=click.IntRange(1, 60),
    help='The number of seconds to wait between each Twitter request')
@click.option(
    '--result-type', '-t',
    type=click.Choice(['recent', 'mixed', 'popular']),
    default='recent',
    help='The result type')
def console(*args, **kwargs):
    run_console(*args, **kwargs)


def main():
    cli()



import threading
import queue


class PrintWorker(threading.Thread):
    def __init__(self, config, q):
        super().__init__()
        self.config = config
        self.queue = q
        self.stopped = False

    def run(self):
        print('Starting the printer worker')
        while not self.stopped:
            try:
                tweet = self.queue.get(timeout=1)
                self.print_tweet(tweet)
            except queue.Empty:
                pass

    @staticmethod
    def print_tweet(tweet):
        print('\n@' + tweet['user']['screen_name'])
        print(tweet['text'])

    def stop(self):
        self.stopped = True



import requests
import base64


def twitter_session(config, session=requests.Session()):
    api_key = config['twitter_oauth']['consumer_key']
    api_secret = config['twitter_oauth']['consumer_secret']
    url = config['twitter_api']['auth_api_url']

    secret = '{}:{}'.format(api_key, api_secret)
    secret64 = base64.b64encode(secret.encode('ascii')).decode('ascii')

    headers = {
        'Authorization': 'Basic {}'.format(secret64),
        'Host': 'api.twitter.com',
    }

    r = session.post(url,
                     headers=headers,
                     data={'grant_type': 'client_credentials'})

    bearer_token = r.json()['access_token']

    def bearer_auth(req):
        req.headers['Authorization'] = 'Bearer ' + bearer_token
        return req

    session.auth = bearer_auth
    return session


def load_tweets(session, config, query, result_type, count=100, since_id=None):
    if count is None:
        count = 100

    query += '-filter:retweets'
    url = config['twitter_api']['tweet_api_url']
    params = {
        'q': query,
        'result_type': result_type,
        'count': count
    }

    if since_id is not None:
        params['since_id'] = since_id

    response = session.get(url, params=params)
    return response.json()['statuses']



import threading
import datetime
import time
from ..twitter import load_tweets, twitter_session


class WallWorker(threading.Thread):
    def __init__(self, config, queue, query, interval, result_type, initial_count=10):
        super().__init__()
        self.config = config
        self.queue = queue
        self.query = query
        self.result_type = result_type
        self.count = initial_count
        self.interval = datetime.timedelta(seconds=interval)
        self.stopped = False
        self.max_tweet_id = None

    def run(self):
        print('Starting the wall worker')
        session = twitter_session(self.config)
        last_request_time = datetime.datetime.fromtimestamp(0)

        while not self.stopped:

            elapsed = datetime.datetime.now() - last_request_time
            if elapsed < self.interval:
                time.sleep(1)
            else:
                self.load_tweets(session)
                last_request_time = datetime.datetime.now()

    def load_tweets(self, session):
        tweets = load_tweets(session, self.config, self.query, self.result_type, self.count, self.max_tweet_id)
        for tw in tweets:
            self.queue.put(tw)

        if len(tweets) > 0:
            self.max_tweet_id = max(tweets, key=lambda t: t['id'])['id']

        self.count = None

    def stop(self):
        self.stopped = True



from setuptools import setup, find_packages

with open('README.txt') as f:
    long_description = ''.join(f.readlines())

setup(
    name='twall',
    version='0.4.1',
    description='Twitter Wall project for MI-PYT by Daniel Maly',
    long_description=long_description,
    author='Daniel Malý',
    author_email='maly.dan@gmail.com',
    keywords='twitter,search,wall',
    license='MIT',
    url='https://github.com/DanielMaly/twall',
    packages=find_packages(),
    include_package_data=True,
    zip_safe=False,
    entry_points={
        'console_scripts': [
            'twall = twall.twall:main',
        ],
    },
    classifiers=[
        'Development Status :: 4 - Beta',
        'Environment :: Console',
        'Environment :: Web Environment',
        'Intended Audience :: End Users/Desktop',
        'Framework :: Flask',
        'Natural Language :: English',
        'Operating System :: MacOS :: MacOS X',
        'Operating System :: POSIX :: Linux',
        'Programming Language :: Python :: 3.5',
        'Programming Language :: Python :: Implementation :: CPython',
        'Programming Language :: Python :: 3 :: Only',
        'Topic :: Internet :: WWW/HTTP :: Dynamic Content',
        'License :: OSI Approved :: MIT License'
    ],
    install_requires=['flask', 'click>=6', 'requests'],
    setup_requires=['pytest-runner>=2.9'],
    tests_require=['pytest==2.9.2', 'betamax>=0.8.0', 'betamax-serializers>=0.2.0']
)



from flask import Flask, Blueprint, render_template, request, Markup
import configparser
from . import twitter_adapter
import os


__location__ = os.path.realpath(os.path.join(os.getcwd(), os.path.dirname(__file__)))

blueprint = Blueprint('app', __name__)


def create_app(auth_filename, debug=False):
    config = configparser.ConfigParser()
    if not os.path.exists(auth_filename):
        raise FileNotFoundError('Authentication file doesn\'t exist. I don\'t have Twitter OAuth data.')

    config.read([auth_filename, os.path.join(__location__, '../settings.cfg')])

    return create_app_with_config(config, debug)


def create_app_with_config(config, debug=False, twitter_session=None):
    app = Flask(__name__)
    app.debug = debug
    twitter_adapter.init(config, twitter_session)

    app.register_blueprint(blueprint)

    app.jinja_env.filters['render_image'] = render_media
    app.jinja_env.filters['render_tweet'] = render_tweet

    return app


@blueprint.route('/')
def wall():
    ctx = {}
    query = request.args.get('q')
    if query is not None:
        result_type = request.args.get('mode').lower()
        ctx['mode'] = result_type
        ctx['query'] = query
        ctx['tweets'] = twitter_adapter.get_tweets(query, result_type=result_type)
    else:
        ctx['mode'] = 'recent'

    return render_template('wall.html', **ctx)


def render_hashtag(hashtag, text):
    return '<a href="https://twitter.com/hashtag/{}" target="_blank">{}</a>'.format(hashtag, text)


def render_mention(screen_name, text):
    return '<a href="https://twitter.com/{}" target="_blank">{}</a>'.format(screen_name, text)


def render_link(url, text):
    return '<a href="{}" target="_blank">{}</a>'.format(url, text)


def render_media(media):
    width = min(400, media['sizes']['medium']['w'])
    height = width * (media['sizes']['medium']['h'] / media['sizes']['medium']['w'])
    return Markup('<img src="{}" width="{}" height="{}"/>'.format(media['media_url'], width, height))


def render_tweet(tweet):
    new_text = tweet['text']

    for mention in tweet['entities']['user_mentions']:
        new_text = enrich_text(tweet['text'], new_text, mention['indices'], mention['screen_name'], render_mention)

    for hashtag in tweet['entities']['hashtags']:
        new_text = enrich_text(tweet['text'], new_text, hashtag['indices'], hashtag['text'], render_hashtag)

    for link in tweet['entities']['urls']:
        new_text = enrich_text(tweet['text'], new_text, link['indices'], link['expanded_url'], render_link)

    return Markup(new_text)


def enrich_text(original_text, constructed_text, indices, entity_info, render_method):
    original_text_chunk = original_text[indices[0]:indices[1]]
    new_text = constructed_text.replace(original_text_chunk, render_method(entity_info,
                                                                           original_text_chunk))
    return new_text
